// IEScript.cpp : Implementation of CIEScript

#include "stdafx.h"

#include "IEScript.h"

#define SCRIPT_PROGID L"VBScript"
#define SCRIPT_EXT _T("VBS")


/////////////////////////////////////////////////////////////////////////////
// CIEScript

HRESULT CIEScript::FinalConstruct()
{
   m_hWnd = NULL;
   m_hwndParent = NULL;
   m_bFocus = FALSE;
   m_dwViewMode = 0;
   m_dwBandID = 0;
   // Find path where DLL is installed
   TCHAR szPath[MAX_PATH];
   ::GetModuleFileName(_Module.GetModuleInstance(), szPath, MAX_PATH);
   LPTSTR p;
   ::GetFullPathName(szPath, MAX_PATH, m_szAppPath, &p);
   if( p!=NULL ) *p = _T('\0');
   return S_OK;
};

void CIEScript::FinalRelease()
{
};


/////////////////////////////////////////////////////////////////////////////
// IOleWindow methods

STDMETHODIMP CIEScript::GetWindow(HWND* phWnd)
{
   ATLASSERT(phWnd);
   if( phWnd==NULL ) return E_POINTER;
   *phWnd = m_hWnd;
   return S_OK;
};

STDMETHODIMP CIEScript::ContextSensitiveHelp(BOOL)
{
   return E_NOTIMPL;
};


/////////////////////////////////////////////////////////////////////////////
// IDockingWindow methods

STDMETHODIMP CIEScript::ShowDW(BOOL fShow)
{
   if(m_hWnd) {
      if(fShow) {
         // Show our window
         ::ShowWindow(m_hWnd, SW_SHOW);
      }
      else {
         // Hide our window
         ::ShowWindow(m_hWnd, SW_HIDE);
      }
   }
   return S_OK;
};

STDMETHODIMP CIEScript::CloseDW(DWORD dwReserved)
{
   ShowDW(FALSE);
   if( ::IsWindow(m_hWnd) ) ::DestroyWindow(m_hWnd);
   m_hWnd = NULL;  
   return S_OK;
};

STDMETHODIMP CIEScript::ResizeBorderDW(LPCRECT prcBorder, IUnknown* punkToolbarSite, BOOL fReserved)
{
   // This method is never called for Band Objects.
   return E_NOTIMPL;
};


/////////////////////////////////////////////////////////////////////////////
// IDeskBand methods

STDMETHODIMP CIEScript::GetBandInfo(DWORD dwBandID, DWORD dwViewMode, DESKBANDINFO* pdbi)
{
   if( pdbi!=NULL ) {
      m_dwBandID = dwBandID;
      m_dwViewMode = dwViewMode;
      if(pdbi->dwMask & DBIM_MINSIZE) {
         if(DBIF_VIEWMODE_FLOATING & dwViewMode) {
            pdbi->ptMinSize.x = SIZE_X;
            pdbi->ptMinSize.y = SIZE_Y;
         }
         else {
            pdbi->ptMinSize.x = SIZE_X;
            pdbi->ptMinSize.y = SIZE_Y;
         }
      }
      if(pdbi->dwMask & DBIM_MAXSIZE) {
         pdbi->ptMaxSize.x = SIZE_X;
         pdbi->ptMaxSize.y = SIZE_Y;
      }
      if(pdbi->dwMask & DBIM_INTEGRAL) {
         pdbi->ptIntegral.x = 1;
         pdbi->ptIntegral.y = 1;
      }
      if(pdbi->dwMask & DBIM_ACTUAL) {
         pdbi->ptActual.x = SIZE_X;
         pdbi->ptActual.y = SIZE_Y;
      }
      if(pdbi->dwMask & DBIM_TITLE) {
         TCHAR szTitle[100];
         ::LoadString(_Module.GetResourceInstance(), IDS_TITLE, szTitle, 100);
         USES_CONVERSION;
         lstrcpyW(pdbi->wszTitle, T2CW(szTitle));
      }
      if(pdbi->dwMask & DBIM_MODEFLAGS) {
         pdbi->dwModeFlags = DBIMF_NORMAL;
      }
      if(pdbi->dwMask & DBIM_BKCOLOR) {
         // Use the default background color by removing this flag.
         pdbi->dwMask &= ~DBIM_BKCOLOR;
      }
      return S_OK;
   }

   return E_INVALIDARG;
};


/////////////////////////////////////////////////////////////////////////////
// IInputObject methods

STDMETHODIMP CIEScript::UIActivateIO(BOOL fActivate, LPMSG pMsg)
{
   if(fActivate) ::SetFocus(m_hWnd);
   return S_OK;
};

STDMETHODIMP CIEScript::HasFocusIO(void)
{
   if(m_bFocus) return S_OK; else return S_FALSE;
};

STDMETHODIMP CIEScript::TranslateAcceleratorIO(LPMSG)
{
   return S_FALSE;
};


/////////////////////////////////////////////////////////////////////////////
// IObjectWithSite methods

STDMETHODIMP CIEScript::SetSite(IUnknown* punkSite)
{
   if( punkSite!=NULL ) {
   
      m_hwndParent = NULL;
      CComQIPtr<IOleWindow> spOleWindow = punkSite;
      if( spOleWindow==NULL ) return E_FAIL;

      if( spOleWindow!=NULL ) {
         spOleWindow->GetWindow(&m_hwndParent);
      };
      if( m_hwndParent==NULL ) return E_FAIL;
      if( !RegisterAndCreateWindow() ) return E_FAIL;

      CComQIPtr<IServiceProvider> spSP(punkSite);
      if( spSP==NULL ) return E_FAIL;

      // Get to the Web Browser app
      HRESULT Hr;
      Hr = spSP->QueryService(IID_IWebBrowserApp, &m_spWebBrowser2);
      if( FAILED(Hr) ) return E_FAIL;
   }
   else {
      if( m_spWebBrowser2!=NULL ) m_spWebBrowser2.Release();
   };

   // Get and keep the IInputObjectSite pointer.
   m_spSite = punkSite;

   return S_OK;
};

STDMETHODIMP CIEScript::GetSite(REFIID riid, LPVOID *ppvReturn)
{
   if( m_spSite ) return m_spSite->QueryInterface(riid, ppvReturn);
   return E_FAIL;
};


/////////////////////////////////////////////////////////////////////////////
// IPersistStream methods

STDMETHODIMP CIEScript::GetClassID(LPCLSID pClassID)
{
   *pClassID = CLSID_IEScript;
   return S_OK;
};

STDMETHODIMP CIEScript::IsDirty(void)
{
   return S_FALSE;
};

STDMETHODIMP CIEScript::Load(LPSTREAM)
{
   return S_OK;
};

STDMETHODIMP CIEScript::Save(LPSTREAM, BOOL)
{
   return S_OK;
};

STDMETHODIMP CIEScript::GetSizeMax(ULARGE_INTEGER*)
{
   return E_NOTIMPL;
};


///////////////////////////////////////////////////////////////////////////
// Window creation

BOOL CIEScript::AddButton(int ID)
{
   TBBUTTON but = { 0 };
   but.iBitmap = 8;
   but.fsStyle = TBSTYLE_DROPDOWN;
   but.fsState = TBSTATE_ENABLED;
   but.idCommand = ID;
   //CString s;
   //s.LoadString(ID);
   //s += _T('\0'); // works, but adding empty string doesn't
   //but.iString = m_ctlToolbar.AddStrings(s);
   but.iString = -1;
   return m_ctlToolbar.InsertButton(-1,&but);
}

BOOL CIEScript::RegisterAndCreateWindow(void)
{
   // If the window doesn't exist yet, create it now.
   if( m_hWnd==NULL ) {
      // Can't create a child window without a parent.
      if( m_hwndParent==NULL ) return FALSE;

      // If the window class has not been registered, then do so.
      WNDCLASS wc;
      if( !::GetClassInfo(_Module.GetModuleInstance(), DB_CLASS_NAME, &wc) ) {
         ZeroMemory(&wc, sizeof(wc));
         wc.style          = CS_GLOBALCLASS;
         wc.lpfnWndProc    = (WNDPROC)WndProc;
         wc.cbClsExtra     = 0;
         wc.cbWndExtra     = 0;
         wc.hInstance      = _Module.GetModuleInstance();
         wc.hIcon          = NULL;
         wc.hCursor        = ::LoadCursor(NULL, IDC_ARROW);
         wc.hbrBackground  = (HBRUSH)COLOR_WINDOW;
         wc.lpszMenuName   = NULL;
         wc.lpszClassName  = DB_CLASS_NAME;
   
         if( !::RegisterClass(&wc) ) {
            // If RegisterClass fails, CreateWindow below will fail.
         }
      }

      // Create the window. The WndProc will set m_hWnd.
      HWND hWnd = ::CreateWindowEx(0,
                      DB_CLASS_NAME,
                      NULL,
                      WS_CHILD | WS_CLIPSIBLINGS,
                      0,
                      0,
                      SIZE_X,
                      SIZE_Y,
                      m_hwndParent,
                      NULL,
                      _Module.GetModuleInstance(),
                      (LPVOID)this);
      if( hWnd==NULL ) {
         ::MessageBeep((UINT)-1);
         return FALSE;
      };

      m_ctlImages.Create(IDB_IMAGES, 16, 1, RGB(0,128,128));

      CRect rc(0,0,SIZE_X,SIZE_Y);
      DWORD dwStyle = WS_CHILD | WS_VISIBLE | TBSTYLE_FLAT | CCS_NODIVIDER;
      m_ctlToolbar.Create(hWnd, &rc, NULL, dwStyle, 0);
      m_ctlToolbar.SetBitmapSize(CSize(16,16));
      m_ctlToolbar.SetImageList(m_ctlImages);
      m_ctlToolbar.SetExtendedStyle(TBSTYLE_EX_DRAWDDARROWS);

      AddButton(ID_CMD_BUTTON1);
      AddButton(ID_CMD_BUTTON2);
      AddButton(ID_CMD_BUTTON3);
      AddButton(ID_CMD_BUTTON4);
      AddButton(ID_CMD_BUTTON5);
   }

   return( NULL != m_hWnd );
}


///////////////////////////////////////////////////////////////////////////
// Private method implementations

LRESULT CALLBACK CIEScript::WndProc(HWND hWnd, UINT uMessage, WPARAM wParam, LPARAM lParam)
{
   CIEScript  *pThis = (CIEScript*)::GetWindowLong(hWnd, GWL_USERDATA);
   switch (uMessage) {
   case WM_NCCREATE:
      {
         LPCREATESTRUCT lpcs = (LPCREATESTRUCT)lParam;
         pThis = (CIEScript*)(lpcs->lpCreateParams);
         ::SetWindowLong(hWnd, GWL_USERDATA, (LONG)pThis);
         // Set the window handle
         pThis->m_hWnd = hWnd;
      }
      break;
   
   case WM_PAINT:
      return pThis->OnPaint();
   
   case WM_COMMAND:
      return pThis->OnCommand(wParam, lParam);
   
   case WM_NOTIFY:
      if( ((LPNMHDR)lParam)->code==TBN_DROPDOWN ) {
         return pThis->OnDropDown((LPNMTOOLBAR)lParam);
      }
      break;

   case WM_SETFOCUS:
      return pThis->OnSetFocus();

   case WM_KILLFOCUS:
      return pThis->OnKillFocus();
   
   case WM_SIZE:
      return pThis->OnSize(lParam);
   }
   return ::DefWindowProc(hWnd, uMessage, wParam, lParam);
}

LRESULT CIEScript::OnCommand(WPARAM wParam, LPARAM lParam)
{
   if( wParam==ID_EDITSCRIPT ) {
      OnEditScript(m_dwPopupID-ID_CMD_BUTTON1);
   }
   else if( wParam>=ID_CMD_BUTTON1 && wParam<=ID_CMD_BUTTON5 ) {
      OnRunScript(wParam-ID_CMD_BUTTON1);
   }
   return 0;
}

LRESULT CIEScript::OnPaint(void)
{
   CPaintDC dc(m_hWnd);
   return 0;
}

LRESULT CIEScript::OnSize(LPARAM lParam)
{
   return 0;
}

void CIEScript::FocusChange(BOOL bFocus)
{
   m_bFocus = bFocus;
   // Inform the input object site that the focus has changed
   if( m_spSite ) m_spSite->OnFocusChangeIS( (IDockingWindow*) this, bFocus);
}

LRESULT CIEScript::OnSetFocus(void)
{
   FocusChange(TRUE);
   return 0;
}

LRESULT CIEScript::OnKillFocus(void)
{
   FocusChange(FALSE);
   return 0;
}

LRESULT CIEScript::OnDropDown(LPNMTOOLBAR lpnmtb)
{
   const int POPUP_MENU_POSITION = 0;

   int ID = lpnmtb->iItem;
   RECT rc;
   m_ctlToolbar.GetRect(ID, &rc);
   ::MapWindowPoints(lpnmtb->hdr.hwndFrom,
                     HWND_DESKTOP, 
                     (LPPOINT)&rc, 
                     2);  

   m_dwPopupID = ID;

   TPMPARAMS tpm;
   tpm.cbSize = sizeof(TPMPARAMS);
   tpm.rcExclude.top    = rc.top;
   tpm.rcExclude.left   = rc.left;
   tpm.rcExclude.bottom = rc.bottom;
   tpm.rcExclude.right  = rc.right;

   CMenu menu;
   menu.LoadMenu(IDR_POPUP);
   CMenuHandle menuPopup = menu.GetSubMenu(POPUP_MENU_POSITION);
   ::SetForegroundWindow(m_hWnd);
   ::TrackPopupMenuEx(
      menuPopup,
      TPM_LEFTALIGN|TPM_LEFTBUTTON|TPM_VERTICAL,
      rc.left,rc.top,
      m_hWnd, 
      &tpm);
   return 0;
}


/////////////////////////////////////////////////////////////////////////////
// Script interfaces

STDMETHODIMP CIEScript::GetItemInfo(LPCOLESTR pstrName,
                                   DWORD dwReturnMask,
                                   IUnknown **ppiunkItem,
                                   ITypeInfo **ppti)
{
   ATLASSERT(pstrName);
   if( (dwReturnMask & SCRIPTINFO_ITYPEINFO)!=0 ) {
      *ppti = NULL;
      return E_FAIL;
   }
   if( (dwReturnMask & SCRIPTINFO_IUNKNOWN)==0 ) return E_FAIL;
   if( ppiunkItem==NULL ) return E_POINTER;
   *ppiunkItem = NULL;
   if( wcscmp( pstrName, L"IE" )==0 ) {
      if( m_spWebBrowser2==NULL ) return E_NOINTERFACE;
      return m_spWebBrowser2->QueryInterface(IID_IUnknown, (LPVOID *)ppiunkItem);
   }
   return E_FAIL;
}

STDMETHODIMP CIEScript::GetDocVersionString(BSTR *pbstrVersion)
{
   if( pbstrVersion==NULL ) return E_POINTER;
   CComBSTR bstr;
   bstr.LoadString(IDS_PROJNAME);
   *pbstrVersion = bstr.Detach();
   return S_OK;
};


/////////////////////////////////////////////////////////////////////////////
// 

void CIEScript::GetScriptFilename(int ID, LPTSTR szPath)
{
   ::wsprintf(szPath, _T("%sSCRIPT%d.%s"), m_szAppPath, ID+1, SCRIPT_EXT);
}

BOOL CIEScript::OnEditScript(int ID)
{
   TCHAR szPath[MAX_PATH];
   GetScriptFilename(ID, szPath);
   ::ShellExecute(m_hWnd, _T("edit"), szPath, NULL, NULL, SW_SHOW);
   return TRUE;
}

BOOL CIEScript::OnRunScript(int ID)
{
   HRESULT hr;

   // Load script file into memory
   CFile f;
   TCHAR szPath[MAX_PATH];
   GetScriptFilename(ID, szPath);
   if( CFile::FileExists(szPath)==FALSE ) return FALSE; // No file
   if( f.Open(szPath)==FALSE ) return FALSE;
   DWORD dwSize = f.GetSize();
   LPSTR pBuffer = new CHAR[dwSize+1];
   if( pBuffer==NULL ) return FALSE; // Out of memory
   f.Read(pBuffer, dwSize);
   pBuffer[dwSize] = '\0';
   f.Close();

   // Convert script to UNICODE
   DWORD nLen = ::lstrlenA(pBuffer);
   LPWSTR pwstr = new WCHAR[nLen+1];
   AtlA2WHelper(pwstr, pBuffer, nLen);
   pwstr[nLen] = L'\0';
   delete [] pBuffer;
   CAutoMemRelease<WCHAR> release;
   release.Attach(pwstr);

   // Create the Active Scripting Engine.
   CComPtr<IActiveScript> spScript;
   hr = spScript.CoCreateInstance(SCRIPT_PROGID);
   if( FAILED(hr) ) return FALSE; // No VBScript?

   CComQIPtr<IActiveScriptSite> spPass(GetUnknown());
   if( spPass==NULL ) return FALSE;

   hr = spScript->SetScriptSite(spPass);
   if( FAILED(hr) ) return FALSE;

   CComQIPtr<IActiveScriptParse> spParse( spScript );
   if( spParse==NULL ) return FALSE;

   spParse->InitNew();

   // Add the object model entries...
   // They are stored as member variables, because we need
   // them in the GetItemInfo() method (see above).
   DWORD dwFlags = SCRIPTITEM_ISVISIBLE | SCRIPTITEM_GLOBALMEMBERS;
   hr = spScript->AddNamedItem(OLESTR("IE"), dwFlags );
   if( FAILED(hr) ) return hr;

   // Start the scripting engine...
   m_spIActiveScript = spScript;
   hr = spScript->SetScriptState(SCRIPTSTATE_STARTED);
   if( FAILED(hr) ) return FALSE;

   // Run the script...
   m_bNoFailures = true;
   hr = spParse->ParseScriptText(pwstr,
                                 NULL,
                                 NULL,
                                 NULL,
                                 0,
                                 0,
                                 0,
                                 NULL,
                                 NULL);

   spScript->SetScriptState(SCRIPTSTATE_CLOSED);

   return TRUE;
}

